"""
MobileMoney API Wrapper
~~~~~~~~~~~~~~~~~~~
A basic wrapper for the MTN MOMO OPEN API.
:copyright: (c) 2022-present rewriteapi
:license: MIT, see LICENSE for more details.
"""

__title__ = 'mobilemoney'
__author__ = 'rewriteapi'
__license__ = 'MIT'
__copyright__ = 'Copyright 2022 rewriteapi'
__version__ = '0.1.2'

from .client import Client
from .errors.errors import *

